:host(.card) {
  outline: 1px solid rgba(0, 0, 0, 0.125);
  border-radius: 0.25rem;
}

:host(.accordion) {
  outline: 1px solid rgba(0, 0, 0, 0.125);
  width: 100%;
}

.card-header {
  align-items: center;
  background-color: rgba(0, 0, 0, 0.03);
  border: unset;
  border-radius: 0.25rem;
  display: inline-flex;
  justify-content: start;
  left: 0;
  outline: unset;
  position: sticky;
  width: 100%;
}

.accordion-header {
  align-items: center;
  background-color: rgba(0, 0, 0, 0.03);
  border: unset;
  outline: 1px solid;
  border-radius: 0;
  display: flex;
  justify-content: start;
  position: sticky;
  left: 0;
  width: 100%;
}

.card-button {
  background-color: transparent;
  margin-left: 0.5em;
  margin-right: 0.5em;
}

.card-header-row {
  margin-left: auto 1em;
  position: relative !important;
  max-width: calc(100% - 2em);
}

.card-title {
  align-items: center;
  font-size: 1.4em;
  font-weight: bold;
  overflow-wrap: break-word;
}

.card-header-row > .bk {
  overflow-wrap: break-word;
  text-align: center;
}
