declare module "styles/editable_slider.css" {
  export const input_group: string
  export const input: string
  export const spin_wrapper: string
  export const spin_btn_up: string
  export const spin_btn_down: string
  export default ""
}
